--- cdr/cdr_pgsql.c.orig2
+++ cdr/cdr_pgsql.c
@@ -59,7 +59,9 @@
 #include "asterisk/cli.h"
 #include "asterisk/module.h"
 
-#define DATE_FORMAT "'%Y-%m-%d %T'"
+#define DATE_FORMAT "%Y-%m-%d %T"
+
+#define PGSQL_MIN_VERSION_SCHEMA 70300
 
 static const char name[] = "pgsql";
 static const char config[] = "cdr_pgsql.conf";
@@ -75,7 +77,8 @@
 static char *tz;
 
 static int connected = 0;
-static int maxsize = 512, maxsize2 = 512;
+/* Optimization to reduce number of memory allocations */
+static int maxsize1 = 512, maxsize2 = 512;
 static time_t connect_time = 0;
 static int totalrecords = 0;
 static int records;
@@ -87,7 +90,7 @@
 
 AST_MUTEX_DEFINE_STATIC(pgsql_lock);
 
-static PGconn	*conn = NULL;
+static PGconn *conn = NULL;
 
 struct columns {
 	char *name;
@@ -95,39 +98,10 @@
 	int len;
 	unsigned int notnull:1;
 	unsigned int hasdefault:1;
-	AST_RWLIST_ENTRY(columns) list;
+	AST_RWLIST_ENTRY(columns) entry;
 };
 
-static AST_RWLIST_HEAD_STATIC(psql_columns, columns);
-
-#define LENGTHEN_BUF1(size)                                               \
-			do {                                                          \
-				/* Lengthen buffer, if necessary */                       \
-				if (ast_str_strlen(sql) + size + 1 > ast_str_size(sql)) { \
-					if (ast_str_make_space(&sql, ((ast_str_size(sql) + size + 3) / 512 + 1) * 512) != 0) {	\
-						ast_log(LOG_ERROR, "Unable to allocate sufficient memory.  Insert CDR failed.\n"); \
-						ast_free(sql);                                    \
-						ast_free(sql2);                                   \
-						AST_RWLIST_UNLOCK(&psql_columns);                 \
-						ast_mutex_unlock(&pgsql_lock);                    \
-						return -1;                                        \
-					}                                                     \
-				}                                                         \
-			} while (0)
-
-#define LENGTHEN_BUF2(size)                               \
-			do {                                          \
-				if (ast_str_strlen(sql2) + size + 1 > ast_str_size(sql2)) {  \
-					if (ast_str_make_space(&sql2, ((ast_str_size(sql2) + size + 3) / 512 + 1) * 512) != 0) {	\
-						ast_log(LOG_ERROR, "Unable to allocate sufficient memory.  Insert CDR failed.\n");	\
-						ast_free(sql);                    \
-						ast_free(sql2);                   \
-						AST_RWLIST_UNLOCK(&psql_columns); \
-						ast_mutex_unlock(&pgsql_lock);    \
-						return -1;                        \
-					}                                     \
-				}                                         \
-			} while (0)
+static AST_LIST_HEAD_STATIC(psql_columns, columns);
 
 /*! \brief Handle the CLI command cdr show pgsql status */
 static char *handle_cdr_pgsql_status(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
@@ -162,6 +136,7 @@
 		if (table && *table) {
 			snprintf(status2, 99, " using table %s", table);
 		}
+
 		if (ctime > 31536000) {
 			ast_cli(a->fd, "%s%s for %d years, %d days, %d hours, %d minutes, %d seconds.\n", status, status2, ctime / 31536000, (ctime % 31536000) / 86400, (ctime % 86400) / 3600, (ctime % 3600) / 60, ctime % 60);
 		} else if (ctime > 86400) {
@@ -179,24 +154,44 @@
 			ast_cli(a->fd, "  Wrote %d records since last restart and %d records since last reconnect.\n", totalrecords, records);
 		}
 	} else {
-		ast_cli(a->fd, "Not currently connected to a PgSQL server.\n");
+		ast_cli(a->fd, "Not currently connected to a PostgreSQL server.\n");
 	}
+
 	return CLI_SUCCESS;
 }
 
-static void pgsql_reconnect(void)
+static void empty_columns(void)
 {
-	struct ast_str *conn_info = ast_str_create(128);
-	if (!conn_info) {
-		ast_log(LOG_ERROR, "Failed to allocate memory for connection string.\n");
-		return;
+	struct columns *current;
+
+	while ((current = AST_LIST_REMOVE_HEAD(&psql_columns, entry))) {
+		ast_free(current);
 	}
+}
 
+static void pgsql_disconnect(void)
+{
 	if (conn) {
 		PQfinish(conn);
 		conn = NULL;
 	}
 
+	connected = 0;
+}
+
+static void pgsql_connect(void)
+{
+	struct ast_str *conn_info = ast_str_create(128);
+	struct columns *cur;
+	char *pgerror;
+
+	if (conn_info == NULL) {
+		ast_log(LOG_ERROR, "Failed to allocate memory for connection string.\n");
+		return;
+	}
+
+	pgsql_disconnect();
+
 	ast_str_set(&conn_info, 0, "host=%s port=%s dbname=%s user=%s",
 		pghostname, pgdbport, pgdbname, pgdbuser);
 
@@ -209,316 +204,391 @@
 	}
 
 	conn = PQconnectdb(ast_str_buffer(conn_info));
-	ast_free(conn_info);
-}
 
-static int pgsql_log(struct ast_cdr *cdr)
-{
-	struct ast_tm tm;
-	char *pgerror;
-	PGresult *result;
-	int res = -1;
+	ast_free(conn_info);
 
-	ast_mutex_lock(&pgsql_lock);
+	if (PQstatus(conn) == CONNECTION_OK) {
+		char sql[768];
+		char *fname, *ftype, *flen, *fnotnull, *fdef;
+		PGresult *result;
+		int i, rows, version;
 
-	if ((!connected) && pghostname && pgdbuser && pgpassword && pgdbname) {
-		pgsql_reconnect();
+		ast_debug(1, "Successfully connected to PostgreSQL database.\n");
+		connected = 1;
+		connect_time = time(NULL);
+		records = 0;
 
-		if (PQstatus(conn) != CONNECTION_BAD) {
-			connected = 1;
-			connect_time = time(NULL);
-			records = 0;
-			if (PQsetClientEncoding(conn, encoding)) {
+		if (PQsetClientEncoding(conn, encoding)) {
 #ifdef HAVE_PGSQL_pg_encoding_to_char
-				ast_log(LOG_WARNING, "Failed to set encoding to '%s'.  Encoding set to default '%s'\n", encoding, pg_encoding_to_char(PQclientEncoding(conn)));
+			ast_log(LOG_WARNING, "Failed to set encoding to '%s'. Encoding set to default '%s'\n", encoding, pg_encoding_to_char(PQclientEncoding(conn)));
 #else
-				ast_log(LOG_WARNING, "Failed to set encoding to '%s'.  Encoding set to default.\n", encoding);
+			ast_log(LOG_WARNING, "Failed to set encoding to '%s'. Encoding set to default.\n", encoding);
 #endif
+		}
+
+		version = PQserverVersion(conn);
+
+		if (version >= PGSQL_MIN_VERSION_SCHEMA) {
+			char *schemaname, *tablename, *tmp_schemaname, *tmp_tablename;
+			size_t len;
+			if (strchr(table, '.')) {
+				tmp_schemaname = ast_strdupa(table);
+				tmp_tablename = strrchr(tmp_schemaname, '.');
+				*tmp_tablename++ = '\0';
+			} else {
+				tmp_schemaname = "";
+				tmp_tablename = table;
 			}
+			len = strlen(tmp_tablename);
+			tablename = ast_alloca(len * 2 + 1);
+			PQescapeStringConn(conn, tablename, tmp_tablename, len, NULL);
+
+			len = strlen(tmp_schemaname);
+			schemaname = ast_alloca(len * 2 + 1);
+			PQescapeStringConn(conn, schemaname, tmp_schemaname, len, NULL);
+
+			snprintf(sql, sizeof(sql), "SELECT a.attname, t.typname, a.attlen, a.attnotnull, d.adsrc, a.atttypmod FROM (((pg_catalog.pg_class c INNER JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace AND c.relname = '%s' AND n.nspname = %s%s%s) INNER JOIN pg_catalog.pg_attribute a ON (NOT a.attisdropped) AND a.attnum > 0 AND a.attrelid = c.oid) INNER JOIN pg_catalog.pg_type t ON t.oid = a.atttypid) LEFT OUTER JOIN pg_attrdef d ON a.atthasdef AND d.adrelid = a.attrelid AND d.adnum = a.attnum ORDER BY n.nspname, c.relname, attnum",
+				tablename,
+				ast_strlen_zero(schemaname) ? "" : "'", ast_strlen_zero(schemaname) ? "current_schema()" : schemaname, ast_strlen_zero(schemaname) ? "" : "'");
 		} else {
-			pgerror = PQerrorMessage(conn);
-			ast_log(LOG_ERROR, "Unable to connect to database server %s.  Calls will not be logged!\n", pghostname);
-			ast_log(LOG_ERROR, "Reason: %s\n", pgerror);
-			PQfinish(conn);
-			conn = NULL;
+			snprintf(sql, sizeof(sql), "SELECT a.attname, t.typname, a.attlen, a.attnotnull, d.adsrc, a.atttypmod FROM pg_class c, pg_type t, pg_attribute a LEFT OUTER JOIN pg_attrdef d ON a.atthasdef AND d.adrelid = a.attrelid AND d.adnum = a.attnum WHERE c.oid = a.attrelid AND a.atttypid = t.oid AND (a.attnum > 0) AND c.relname = '%s' ORDER BY c.relname, attnum", table);
+		}
+
+		/* Query the columns */
+		result = PQexec(conn, sql);
+		if (PQresultStatus(result) != PGRES_TUPLES_OK) {
+			pgerror = PQresultErrorMessage(result);
+			ast_log(LOG_ERROR, "Failed to query database columns: %s\n", pgerror);
+			PQclear(result);
+			pgsql_disconnect();
+			return;
+		}
+
+		rows = PQntuples(result);
+		if (rows == 0) {
+			ast_log(LOG_ERROR, "Failed to query database columns. No columns found, does the table exist?\n");
+			PQclear(result);
+			pgsql_disconnect();
+			return;
+		}
+
+		/* Clear out the columns list. */
+		empty_columns();
+
+		for (i = 0; i < rows; i++) {
+			fname = PQgetvalue(result, i, 0);
+			ftype = PQgetvalue(result, i, 1);
+			flen = PQgetvalue(result, i, 2);
+			fnotnull = PQgetvalue(result, i, 3);
+			fdef = PQgetvalue(result, i, 4);
+			if (atoi(flen) == -1) {
+				/* For varchar columns, the maximum length is encoded in a different field */
+				flen = PQgetvalue(result, i, 5);
+			}
+
+			ast_verb(4, "Found column '%s' of type '%s'\n", fname, ftype);
+
+			cur = ast_calloc(1, sizeof(*cur) + strlen(fname) + strlen(ftype) + 2);
+			if (cur) {
+				sscanf(flen, "%30d", &cur->len);
+				cur->name = (char *)cur + sizeof(*cur);
+				cur->type = (char *)cur + sizeof(*cur) + strlen(fname) + 1;
+				strcpy(cur->name, fname);
+				strcpy(cur->type, ftype);
+				if (*fnotnull == 't') {
+					cur->notnull = 1;
+				} else {
+					cur->notnull = 0;
+				}
+				if (!ast_strlen_zero(fdef)) {
+					cur->hasdefault = 1;
+				} else {
+					cur->hasdefault = 0;
+				}
+				AST_LIST_INSERT_TAIL(&psql_columns, cur, entry);
+			}
 		}
+
+		PQclear(result);
+	} else {
+		pgerror = PQerrorMessage(conn);
+		ast_log(LOG_ERROR, "Unable to connect to database server %s. Calls will not be logged!\n", pghostname);
+		ast_log(LOG_ERROR, "Reason: %s\n", pgerror);
+		pgsql_disconnect();
+	}
+}
+
+#define LENGTHEN_BUF(size, var) \
+	do { \
+		/* Lengthen buffer, if necessary */ \
+		if (ast_str_strlen(var) + size + 1 > ast_str_size(var)) { \
+			if (ast_str_make_space(&var, ((ast_str_size(var) + size + 3) / 512 + 1) * 512) != 0) { \
+				ast_log(LOG_ERROR, "Unable to allocate sufficient memory. Insert CDR '%s:%s' failed.\n", pghostname, table); \
+				res = -1; \
+				goto cleanup; \
+			} \
+		} \
+	} while (0)
+
+#define LENGTHEN_BUF1(size) \
+	LENGTHEN_BUF(size, sql1)
+#define LENGTHEN_BUF2(size) \
+	LENGTHEN_BUF(size, sql2)
+
+static int pgsql_log(struct ast_cdr *cdr)
+{
+	int res = 0;
+
+	ast_mutex_lock(&pgsql_lock);
+
+	if (!connected) {
+		pgsql_connect();
 	}
 
 	if (connected) {
+		struct ast_tm tm;
 		struct columns *cur;
-		struct ast_str *sql = ast_str_create(maxsize), *sql2 = ast_str_create(maxsize2);
-		char buf[257];
-		char *escapebuf = NULL, *value;
-		int first = 1;
-		size_t bufsize = 513;
-
-		escapebuf = ast_malloc(bufsize);
-		if (!escapebuf || !sql || !sql2) {
-			goto ast_log_cleanup;
+		struct ast_str *sql1, *sql2;
+		char valbuf[256];
+		char *value;
+		char *separator = "";
+		PGresult *result;
+		char *pgerror;
+
+		/* Test to be sure we're still connected... */
+		/* If we're connected, and connection is working, good. */
+		/* Otherwise, attempt reconnect.  If it fails... sorry... */
+		PQconsumeInput(conn);
+		if (PQstatus(conn) != CONNECTION_OK) {
+			ast_log(LOG_WARNING, "Connection was lost... attempting to reconnect.\n");
+			pgsql_connect();
+			if (!connected) {
+				ast_mutex_unlock(&pgsql_lock);
+				return -1;
+			}
 		}
 
-		ast_str_set(&sql, 0, "INSERT INTO %s (", table);
-		ast_str_set(&sql2, 0, " VALUES (");
+		sql1 = ast_str_create(maxsize1);
+		sql2 = ast_str_create(maxsize2);
 
-		AST_RWLIST_RDLOCK(&psql_columns);
-		AST_RWLIST_TRAVERSE(&psql_columns, cur, list) {
-			/* For fields not set, simply skip them */
-			ast_cdr_format_var(cdr, cur->name, &value, buf, sizeof(buf), 0);
-			if (strcmp(cur->name, "calldate") == 0 && !value) {
-				ast_cdr_format_var(cdr, "start", &value, buf, sizeof(buf), 0);
+		if (sql1 == NULL || sql2 == NULL) {
+			ast_mutex_unlock(&pgsql_lock);
+			ast_free(sql1);
+			ast_free(sql2);
+			return -1;
+		}
+
+		ast_str_set(&sql1,  0, "INSERT INTO %s (", table);
+		ast_str_set(&sql2, 0, "VALUES (");
+
+		AST_LIST_TRAVERSE(&psql_columns, cur, entry) {
+			ast_cdr_format_var(cdr, cur->name, &value, valbuf, sizeof(valbuf), 0);
+
+			if (value == NULL && strcmp(cur->name, "calldate") == 0) {
+				ast_cdr_format_var(cdr, "start", &value, valbuf, sizeof(valbuf), 0);
 			}
-			if (!value) {
+
+			/* For fields not set, simply skip them */
+			if (value == NULL) {
+				/* Field is NOT NULL (but no default), must include it anyway */
 				if (cur->notnull && !cur->hasdefault) {
-					/* Field is NOT NULL (but no default), must include it anyway */
-					LENGTHEN_BUF1(strlen(cur->name) + 2);
-					ast_str_append(&sql, 0, "%s\"%s\"", first ? "" : ",", cur->name);
-					LENGTHEN_BUF2(3);
-					ast_str_append(&sql2, 0, "%s''", first ? "" : ",");
-					first = 0;
+					if (strncmp(cur->type, "int", 3) == 0 || strncmp(cur->type, "float", 5) == 0) {
+						LENGTHEN_BUF2(2);
+						ast_str_append(&sql2, 0, "%s0", separator);
+					} else {
+						LENGTHEN_BUF2(3);
+						ast_str_append(&sql2, 0, "%s''", separator);
+					}
+
+					LENGTHEN_BUF1(strlen(cur->name) + 3);
+					ast_str_append(&sql1, 0, "%s\"%s\"", separator, cur->name);
+					separator = ",";
 				}
+
 				continue;
 			}
 
-			LENGTHEN_BUF1(strlen(cur->name) + 2);
-			ast_str_append(&sql, 0, "%s\"%s\"", first ? "" : ",", cur->name);
-
 			if (strcmp(cur->name, "start") == 0 || strcmp(cur->name, "calldate") == 0) {
 				if (strncmp(cur->type, "int", 3) == 0) {
 					LENGTHEN_BUF2(13);
-					ast_str_append(&sql2, 0, "%s%ld", first ? "" : ",", (long) cdr->start.tv_sec);
+					ast_str_append(&sql2, 0, "%s%ld", separator, (long) cdr->start.tv_sec);
 				} else if (strncmp(cur->type, "float", 5) == 0) {
 					LENGTHEN_BUF2(31);
-					ast_str_append(&sql2, 0, "%s%f", first ? "" : ",", (double)cdr->start.tv_sec + (double)cdr->start.tv_usec / 1000000.0);
+					ast_str_append(&sql2, 0, "%s%f", separator,
+						(double) cdr->start.tv_sec +
+						(double) cdr->start.tv_usec / 1000000.0);
 				} else {
 					/* char, hopefully */
 					LENGTHEN_BUF2(31);
 					ast_localtime(&cdr->start, &tm, tz);
-					ast_strftime(buf, sizeof(buf), DATE_FORMAT, &tm);
-					ast_str_append(&sql2, 0, "%s%s", first ? "" : ",", buf);
+					ast_strftime(valbuf, sizeof(valbuf), DATE_FORMAT, &tm);
+					ast_str_append(&sql2, 0, "%s'%s'", separator, valbuf);
 				}
 			} else if (strcmp(cur->name, "answer") == 0) {
 				if (strncmp(cur->type, "int", 3) == 0) {
 					LENGTHEN_BUF2(13);
-					ast_str_append(&sql2, 0, "%s%ld", first ? "" : ",", (long) cdr->answer.tv_sec);
+					ast_str_append(&sql2, 0, "%s%ld", separator, (long) cdr->answer.tv_sec);
 				} else if (strncmp(cur->type, "float", 5) == 0) {
 					LENGTHEN_BUF2(31);
-					ast_str_append(&sql2, 0, "%s%f", first ? "" : ",", (double)cdr->answer.tv_sec + (double)cdr->answer.tv_usec / 1000000.0);
+					ast_str_append(&sql2, 0, "%s%f", separator,
+						(double) cdr->answer.tv_sec +
+						(double) cdr->answer.tv_usec / 1000000.0);
 				} else {
 					/* char, hopefully */
 					LENGTHEN_BUF2(31);
 					ast_localtime(&cdr->answer, &tm, tz);
-					ast_strftime(buf, sizeof(buf), DATE_FORMAT, &tm);
-					ast_str_append(&sql2, 0, "%s%s", first ? "" : ",", buf);
+					ast_strftime(valbuf, sizeof(valbuf), DATE_FORMAT, &tm);
+					ast_str_append(&sql2, 0, "%s%s", separator, valbuf);
 				}
 			} else if (strcmp(cur->name, "end") == 0) {
 				if (strncmp(cur->type, "int", 3) == 0) {
 					LENGTHEN_BUF2(13);
-					ast_str_append(&sql2, 0, "%s%ld", first ? "" : ",", (long) cdr->end.tv_sec);
+					ast_str_append(&sql2, 0, "%s%ld", separator, (long) cdr->end.tv_sec);
 				} else if (strncmp(cur->type, "float", 5) == 0) {
 					LENGTHEN_BUF2(31);
-					ast_str_append(&sql2, 0, "%s%f", first ? "" : ",", (double)cdr->end.tv_sec + (double)cdr->end.tv_usec / 1000000.0);
+					ast_str_append(&sql2, 0, "%s%f", separator,
+						(double) cdr->end.tv_sec +
+						(double) cdr->end.tv_usec / 1000000.0);
 				} else {
 					/* char, hopefully */
 					LENGTHEN_BUF2(31);
 					ast_localtime(&cdr->end, &tm, tz);
-					ast_strftime(buf, sizeof(buf), DATE_FORMAT, &tm);
-					ast_str_append(&sql2, 0, "%s%s", first ? "" : ",", buf);
+					ast_strftime(valbuf, sizeof(valbuf), DATE_FORMAT, &tm);
+					ast_str_append(&sql2, 0, "%s%s", separator, valbuf);
 				}
 			} else if (strcmp(cur->name, "duration") == 0 || strcmp(cur->name, "billsec") == 0) {
-				if (cur->type[0] == 'i') {
+				if (strncmp(cur->type, "int", 3) == 0) {
 					/* Get integer, no need to escape anything */
-					ast_cdr_format_var(cdr, cur->name, &value, buf, sizeof(buf), 0);
 					LENGTHEN_BUF2(13);
-					ast_str_append(&sql2, 0, "%s%s", first ? "" : ",", value);
-				} else if (strncmp(cur->type, "float", 5) == 0) {
-					struct timeval *when = cur->name[0] == 'd' ? &cdr->start : ast_tvzero(cdr->answer) ? &cdr->end : &cdr->answer;
-					LENGTHEN_BUF2(31);
-					ast_str_append(&sql2, 0, "%s%f", first ? "" : ",", (double) (ast_tvdiff_us(cdr->end, *when) / 1000000.0));
+					ast_str_append(&sql2, 0, "%s%s", separator, value);
 				} else {
-					/* Char field, probably */
 					struct timeval *when = cur->name[0] == 'd' ? &cdr->start : ast_tvzero(cdr->answer) ? &cdr->end : &cdr->answer;
 					LENGTHEN_BUF2(31);
-					ast_str_append(&sql2, 0, "%s'%f'", first ? "" : ",", (double) (ast_tvdiff_us(cdr->end, *when) / 1000000.0));
+					if (strncmp(cur->type, "float", 5) == 0) {
+						ast_str_append(&sql2, 0, "%s%f", separator, (double) (ast_tvdiff_us(cdr->end, *when) / 1000000.0));
+					} else {
+						/* Char field, probably */
+						ast_str_append(&sql2, 0, "%s'%f'", separator, (double) (ast_tvdiff_us(cdr->end, *when) / 1000000.0));
+					}
 				}
 			} else if (strcmp(cur->name, "disposition") == 0 || strcmp(cur->name, "amaflags") == 0) {
 				if (strncmp(cur->type, "int", 3) == 0) {
 					/* Integer, no need to escape anything */
-					ast_cdr_format_var(cdr, cur->name, &value, buf, sizeof(buf), 1);
+					ast_cdr_format_var(cdr, cur->name, &value, valbuf, sizeof(valbuf), 1);
 					LENGTHEN_BUF2(13);
-					ast_str_append(&sql2, 0, "%s%s", first ? "" : ",", value);
+					ast_str_append(&sql2, 0, "%s%s", separator, value);
 				} else {
 					/* Although this is a char field, there are no special characters in the values for these fields */
-					ast_cdr_format_var(cdr, cur->name, &value, buf, sizeof(buf), 0);
 					LENGTHEN_BUF2(31);
-					ast_str_append(&sql2, 0, "%s'%s'", first ? "" : ",", value);
+					ast_str_append(&sql2, 0, "%s'%s'", separator, value);
 				}
 			} else {
 				/* Arbitrary field, could be anything */
-				ast_cdr_format_var(cdr, cur->name, &value, buf, sizeof(buf), 0);
 				if (strncmp(cur->type, "int", 3) == 0) {
 					long long whatever;
-					if (value && sscanf(value, "%30lld", &whatever) == 1) {
+					if (sscanf(value, "%30lld", &whatever) == 1) {
 						LENGTHEN_BUF2(26);
-						ast_str_append(&sql2, 0, "%s%lld", first ? "" : ",", whatever);
+						ast_str_append(&sql2, 0, "%s%lld", separator, whatever);
 					} else {
-						LENGTHEN_BUF2(2);
-						ast_str_append(&sql2, 0, "%s0", first ? "" : ",");
+						value = NULL;
 					}
 				} else if (strncmp(cur->type, "float", 5) == 0) {
 					long double whatever;
-					if (value && sscanf(value, "%30Lf", &whatever) == 1) {
+					if (sscanf(value, "%30Lf", &whatever) == 1) {
 						LENGTHEN_BUF2(51);
-						ast_str_append(&sql2, 0, "%s%30Lf", first ? "" : ",", whatever);
+						ast_str_append(&sql2, 0, "%s%30Lf", separator, whatever);
 					} else {
-						LENGTHEN_BUF2(2);
-						ast_str_append(&sql2, 0, "%s0", first ? "" : ",");
+						value = NULL;
 					}
-				/* XXX Might want to handle dates, times, and other misc fields here XXX */
 				} else {
-					if (value) {
-						size_t required_size = strlen(value) * 2 + 1;
-
-						/* If our argument size exceeds our buffer, grow it,
-						 * as PQescapeStringConn() expects the buffer to be
-						 * adequitely sized and does *NOT* do size checking.
-						 */
-						if (required_size > bufsize) {
-							char *tmpbuf = ast_realloc(escapebuf, required_size);
-
-							if (!tmpbuf) {
-								AST_RWLIST_UNLOCK(&psql_columns);
-								goto ast_log_cleanup;
-							}
+					size_t len = strlen(value);
+					char *escapebuf = ast_alloca(len * 2 + 1);
+					len = PQescapeStringConn(conn, escapebuf, value, len, NULL);
+					LENGTHEN_BUF2(len + 3);
+					ast_str_append(&sql2, 0, "%s'%s'", separator, escapebuf);
+				}
 
-							escapebuf = tmpbuf;
-							bufsize = required_size;
+				if (value == NULL) {
+					if (cur->notnull && !cur->hasdefault) {
+						if (strncmp(cur->type, "int", 3) == 0 || strncmp(cur->type, "float", 5) == 0) {
+							LENGTHEN_BUF2(2);
+							ast_str_append(&sql2, 0, "%s0", separator);
+						} else {
+							LENGTHEN_BUF2(3);
+							ast_str_append(&sql2, 0, "%s''", separator);
 						}
-						PQescapeStringConn(conn, escapebuf, value, strlen(value), NULL);
 					} else {
-						escapebuf[0] = '\0';
+						continue;
 					}
-					LENGTHEN_BUF2(strlen(escapebuf) + 3);
-					ast_str_append(&sql2, 0, "%s'%s'", first ? "" : ",", escapebuf);
 				}
 			}
-			first = 0;
+
+			LENGTHEN_BUF1(strlen(cur->name) + 3);
+			ast_str_append(&sql1, 0, "%s\"%s\"", separator, cur->name);
+			separator = ",";
 		}
 
-		LENGTHEN_BUF1(ast_str_strlen(sql2) + 2);
-		AST_RWLIST_UNLOCK(&psql_columns);
-		ast_str_append(&sql, 0, ")%s)", ast_str_buffer(sql2));
+		LENGTHEN_BUF1(ast_str_strlen(sql2) + 3);
+		ast_str_append(&sql1, 0, ") %s)", ast_str_buffer(sql2));
 
-		ast_debug(3, "Inserting a CDR record: [%s]\n", ast_str_buffer(sql));
+		ast_debug(3, "Inserting record: [%s]\n", ast_str_buffer(sql1));
 
-		/* Test to be sure we're still connected... */
-		/* If we're connected, and connection is working, good. */
-		/* Otherwise, attempt reconnect.  If it fails... sorry... */
-		if (PQstatus(conn) == CONNECTION_OK) {
-			connected = 1;
-		} else {
-			ast_log(LOG_ERROR, "Connection was lost... attempting to reconnect.\n");
-			PQreset(conn);
-			if (PQstatus(conn) == CONNECTION_OK) {
-				ast_log(LOG_ERROR, "Connection reestablished.\n");
-				connected = 1;
-				connect_time = time(NULL);
-				records = 0;
-			} else {
-				pgerror = PQerrorMessage(conn);
-				ast_log(LOG_ERROR, "Unable to reconnect to database server %s. Calls will not be logged!\n", pghostname);
-				ast_log(LOG_ERROR, "Reason: %s\n", pgerror);
-				PQfinish(conn);
-				conn = NULL;
-				connected = 0;
-				goto ast_log_cleanup;
-			}
-		}
-		result = PQexec(conn, ast_str_buffer(sql));
+		result = PQexec(conn, ast_str_buffer(sql1));
 		if (PQresultStatus(result) != PGRES_COMMAND_OK) {
 			pgerror = PQresultErrorMessage(result);
-			ast_log(LOG_ERROR, "Failed to insert call detail record into database!\n");
-			ast_log(LOG_ERROR, "Reason: %s\n", pgerror);
-			ast_log(LOG_ERROR, "Connection may have been lost... attempting to reconnect.\n");
+			ast_log(LOG_WARNING, "Failed insert record into database!\n");
+			ast_log(LOG_WARNING, "Reason: %s\n", pgerror);
+			ast_log(LOG_WARNING, "Connection may have been lost... attempting to reconnect.\n");
 			PQreset(conn);
 			if (PQstatus(conn) == CONNECTION_OK) {
-				ast_log(LOG_ERROR, "Connection reestablished.\n");
-				connected = 1;
+				ast_log(LOG_NOTICE, "Connection reestablished.\n");
 				connect_time = time(NULL);
 				records = 0;
 				PQclear(result);
-				result = PQexec(conn, ast_str_buffer(sql));
+				result = PQexec(conn, ast_str_buffer(sql1));
 				if (PQresultStatus(result) != PGRES_COMMAND_OK) {
 					pgerror = PQresultErrorMessage(result);
-					ast_log(LOG_ERROR, "HARD ERROR!  Attempted reconnection failed.  DROPPING CALL RECORD!\n");
+					ast_log(LOG_ERROR, "HARD ERROR! DROPPING RECORD!\n");
 					ast_log(LOG_ERROR, "Reason: %s\n", pgerror);
+					pgsql_disconnect();
+					res = -1;
 				} else {
-					/* Second try worked out ok */
 					totalrecords++;
 					records++;
-					res = 0;
 				}
+			} else {
+				pgerror = PQerrorMessage(conn);
+				ast_log(LOG_ERROR, "Unable to reconnect to database server %s. Call will not be logged!\n", pghostname);
+				ast_log(LOG_ERROR, "Reason: %s\n", pgerror);
+				pgsql_disconnect();
+				res = -1;
 			}
 		} else {
 			totalrecords++;
 			records++;
-			res = 0;
 		}
 		PQclear(result);
 
-ast_log_cleanup:
-		ast_free(escapebuf);
-		ast_free(sql);
+		/* Next time, just allocate buffers that are that big to start with. */
+		if (ast_str_strlen(sql1) > maxsize1) {
+			maxsize1 = ast_str_strlen(sql1);
+		}
+		if (ast_str_strlen(sql2) > maxsize2) {
+			maxsize2 = ast_str_strlen(sql2);
+		}
+
+cleanup:
+		ast_free(sql1);
 		ast_free(sql2);
 	}
 
 	ast_mutex_unlock(&pgsql_lock);
-	return res;
-}
-
-/* This function should be called without holding the pgsql_columns lock */
-static void empty_columns(void)
-{
-	struct columns *current;
-	AST_RWLIST_WRLOCK(&psql_columns);
-	while ((current = AST_RWLIST_REMOVE_HEAD(&psql_columns, list))) {
-		ast_free(current);
-	}
-	AST_RWLIST_UNLOCK(&psql_columns);
-
-}
 
-static int unload_module(void)
-{
-	if (ast_cdr_unregister(name)) {
-		return -1;
-	}
-
-	ast_cli_unregister_multiple(cdr_pgsql_status_cli, ARRAY_LEN(cdr_pgsql_status_cli));
-
-	if (conn) {
-		PQfinish(conn);
-		conn = NULL;
-	}
-	ast_free(pghostname);
-	ast_free(pgdbname);
-	ast_free(pgdbuser);
-	ast_free(pgpassword);
-	ast_free(pgappname);
-	ast_free(pgdbport);
-	ast_free(table);
-	ast_free(encoding);
-	ast_free(tz);
-
-	empty_columns();
-
-	return 0;
+	return res;
 }
 
 static int config_module(int reload)
 {
-	char *pgerror;
-	struct columns *cur;
-	PGresult *result;
 	const char *tmp;
 	struct ast_config *cfg;
 	struct ast_flags config_flags = { reload ? CONFIG_FLAG_FILEUNCHANGED : 0 };
@@ -533,270 +603,181 @@
 	ast_mutex_lock(&pgsql_lock);
 
 	if (!ast_variable_browse(cfg, "global")) {
-		ast_config_destroy(cfg);
-		ast_mutex_unlock(&pgsql_lock);
-		ast_log(LOG_NOTICE, "cdr_pgsql configuration contains no global section, skipping module %s.\n",
-			reload ? "reload" : "load");
-		return -1;
+		ast_log(LOG_NOTICE, "%s config file missing global section, skipping module %s.\n",
+			config, reload ? "reload" : "load");
+		goto err;
 	}
 
-	if (!(tmp = ast_variable_retrieve(cfg, "global", "hostname"))) {
-		ast_log(LOG_WARNING, "PostgreSQL server hostname not specified.  Assuming unix socket connection\n");
+	if (!(tmp = ast_variable_retrieve(cfg,"global","hostname"))) {
+		ast_log(LOG_WARNING,"PostgreSQL server hostname not specified. Assuming unix socket connection\n");
 		tmp = "";	/* connect via UNIX-socket by default */
 	}
-
 	ast_free(pghostname);
 	if (!(pghostname = ast_strdup(tmp))) {
-		ast_config_destroy(cfg);
-		ast_mutex_unlock(&pgsql_lock);
-		return -1;
+		ast_log(LOG_WARNING,"Insufficient memory for copying hostname info\n\n");
+		goto err;
 	}
 
-	if (!(tmp = ast_variable_retrieve(cfg, "global", "dbname"))) {
-		ast_log(LOG_WARNING, "PostgreSQL database not specified.  Assuming asterisk\n");
-		tmp = "asteriskcdrdb";
+	if (!(tmp = ast_variable_retrieve(cfg,"global","port"))) {
+		ast_log(LOG_WARNING,"PostgreSQL database port not specified. Using default 5432.\n");
+		tmp = "5432";
+	}
+	ast_free(pgdbport);
+	if (!(pgdbport = ast_strdup(tmp))) {
+		ast_log(LOG_WARNING,"Insufficient memory for copying port info\n");
+		goto err;
 	}
 
+	if (!(tmp = ast_variable_retrieve(cfg, "global", "dbname"))) {
+		ast_log(LOG_WARNING,"PostgreSQL database not specified. Assuming asterisk\n");
+		tmp = "asterisk";
+	}
 	ast_free(pgdbname);
 	if (!(pgdbname = ast_strdup(tmp))) {
-		ast_config_destroy(cfg);
-		ast_mutex_unlock(&pgsql_lock);
-		return -1;
+		ast_log(LOG_WARNING,"Insufficient memory for copying dbname info\n");
+		goto err;
 	}
 
 	if (!(tmp = ast_variable_retrieve(cfg, "global", "user"))) {
-		ast_log(LOG_WARNING, "PostgreSQL database user not specified.  Assuming asterisk\n");
+		ast_log(LOG_WARNING,"PostgreSQL database user not specified. Assuming asterisk\n");
 		tmp = "asterisk";
 	}
-
 	ast_free(pgdbuser);
 	if (!(pgdbuser = ast_strdup(tmp))) {
-		ast_config_destroy(cfg);
-		ast_mutex_unlock(&pgsql_lock);
-		return -1;
-	}
-
-	if (!(tmp = ast_variable_retrieve(cfg, "global", "appname"))) {
-		tmp = "";
+		ast_log(LOG_WARNING,"Insufficient memory for copying user info\n");
+		goto err;
 	}
 
-	ast_free(pgappname);
-	if (!(pgappname = ast_strdup(tmp))) {
-		ast_config_destroy(cfg);
-		ast_mutex_unlock(&pgsql_lock);
-		return -1;
-	}
-
-
 	if (!(tmp = ast_variable_retrieve(cfg, "global", "password"))) {
-		ast_log(LOG_WARNING, "PostgreSQL database password not specified.  Assuming blank\n");
+		ast_log(LOG_WARNING, "PostgreSQL database password not specified. Assuming blank\n");
 		tmp = "";
 	}
-
 	ast_free(pgpassword);
 	if (!(pgpassword = ast_strdup(tmp))) {
-		ast_config_destroy(cfg);
-		ast_mutex_unlock(&pgsql_lock);
-		return -1;
-	}
-
-	if (!(tmp = ast_variable_retrieve(cfg, "global", "port"))) {
-		ast_log(LOG_WARNING, "PostgreSQL database port not specified.  Using default 5432.\n");
-		tmp = "5432";
-	}
-
-	ast_free(pgdbport);
-	if (!(pgdbport = ast_strdup(tmp))) {
-		ast_config_destroy(cfg);
-		ast_mutex_unlock(&pgsql_lock);
-		return -1;
+		ast_log(LOG_WARNING,"Insufficient memory for copying password info\n");
+		goto err;
 	}
 
 	if (!(tmp = ast_variable_retrieve(cfg, "global", "table"))) {
-		ast_log(LOG_WARNING, "CDR table not specified.  Assuming cdr\n");
-		tmp = "cdr";
+		ast_log(LOG_WARNING,"CEL table not specified. Assuming cel\n");
+		tmp = "cel";
 	}
-
 	ast_free(table);
 	if (!(table = ast_strdup(tmp))) {
-		ast_config_destroy(cfg);
-		ast_mutex_unlock(&pgsql_lock);
-		return -1;
+		ast_log(LOG_WARNING,"Insufficient memory for copying table info\n");
+		goto err;
 	}
 
-	if (!(tmp = ast_variable_retrieve(cfg, "global", "encoding"))) {
-		ast_log(LOG_WARNING, "Encoding not specified.  Assuming LATIN9\n");
-		tmp = "LATIN9";
+	if (!(tmp = ast_variable_retrieve(cfg, "global", "appname"))) {
+		tmp = "";
+	}
+	ast_free(pgappname);
+	if (!(pgappname = ast_strdup(tmp))) {
+		ast_log(LOG_WARNING,"Insufficient memory for copying appname info\n");
+		goto err;
 	}
 
+	if (!(tmp = ast_variable_retrieve(cfg, "global", "encoding"))) {
+		ast_log(LOG_WARNING, "Encoding not specified. Assuming UTF8\n");
+		tmp = "UTF8";
+	}
 	ast_free(encoding);
 	if (!(encoding = ast_strdup(tmp))) {
-		ast_config_destroy(cfg);
-		ast_mutex_unlock(&pgsql_lock);
-		return -1;
+		ast_log(LOG_WARNING,"Insufficient memory for copying encoding info\n");
+		goto err;
 	}
 
 	if (!(tmp = ast_variable_retrieve(cfg, "global", "timezone"))) {
 		tmp = "";
 	}
-
 	ast_free(tz);
 	tz = NULL;
-
 	if (!ast_strlen_zero(tmp) && !(tz = ast_strdup(tmp))) {
-		ast_config_destroy(cfg);
-		ast_mutex_unlock(&pgsql_lock);
-		return -1;
+		ast_log(LOG_WARNING,"Insufficient memory for copying timezone info\n");
+		goto err;
 	}
 
 	if (option_debug) {
 		if (ast_strlen_zero(pghostname)) {
-			ast_debug(1, "using default unix socket\n");
+			ast_debug(3, "Using default unix socket\n");
 		} else {
-			ast_debug(1, "got hostname of %s\n", pghostname);
+			ast_debug(3, "Got hostname of %s\n", pghostname);
 		}
-		ast_debug(1, "got port of %s\n", pgdbport);
-		ast_debug(1, "got user of %s\n", pgdbuser);
-		ast_debug(1, "got dbname of %s\n", pgdbname);
-		ast_debug(1, "got password of %s\n", pgpassword);
-		ast_debug(1, "got application name of %s\n", pgappname);
-		ast_debug(1, "got sql table name of %s\n", table);
-		ast_debug(1, "got encoding of %s\n", encoding);
-		ast_debug(1, "got timezone of %s\n", tz);
+		ast_debug(3, "Got port of %s\n", pgdbport);
+		ast_debug(3, "Got dbname of %s\n", pgdbname);
+		ast_debug(3, "Got user of %s\n", pgdbuser);
+		ast_debug(3, "Got password of %s\n", pgpassword);
+		ast_debug(3, "Got table name of %s\n", table);
+		ast_debug(3, "Got application name of %s\n", pgappname);
+		ast_debug(3, "Got encoding of %s\n", encoding);
+		ast_debug(3, "Got timezone of %s\n", tz);
 	}
 
-	pgsql_reconnect();
-
-	if (PQstatus(conn) != CONNECTION_BAD) {
-		char sqlcmd[768];
-		char *fname, *ftype, *flen, *fnotnull, *fdef;
-		int i, rows, version;
-		ast_debug(1, "Successfully connected to PostgreSQL database.\n");
-		connected = 1;
-		connect_time = time(NULL);
-		records = 0;
-		if (PQsetClientEncoding(conn, encoding)) {
-#ifdef HAVE_PGSQL_pg_encoding_to_char
-			ast_log(LOG_WARNING, "Failed to set encoding to '%s'.  Encoding set to default '%s'\n", encoding, pg_encoding_to_char(PQclientEncoding(conn)));
-#else
-			ast_log(LOG_WARNING, "Failed to set encoding to '%s'.  Encoding set to default.\n", encoding);
-#endif
-		}
-		version = PQserverVersion(conn);
-
-		if (version >= 70300) {
-			char *schemaname, *tablename, *tmp_schemaname, *tmp_tablename;
-			if (strchr(table, '.')) {
-				tmp_schemaname = ast_strdupa(table);
-				tmp_tablename = strchr(tmp_schemaname, '.');
-				*tmp_tablename++ = '\0';
-			} else {
-				tmp_schemaname = "";
-				tmp_tablename = table;
-			}
-			tablename = ast_alloca(strlen(tmp_tablename) * 2 + 1);
-			PQescapeStringConn(conn, tablename, tmp_tablename, strlen(tmp_tablename), NULL);
-
-			schemaname = ast_alloca(strlen(tmp_schemaname) * 2 + 1);
-			PQescapeStringConn(conn, schemaname, tmp_schemaname, strlen(tmp_schemaname), NULL);
-
-			snprintf(sqlcmd, sizeof(sqlcmd), "SELECT a.attname, t.typname, a.attlen, a.attnotnull, d.adsrc, a.atttypmod FROM (((pg_catalog.pg_class c INNER JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace AND c.relname = '%s' AND n.nspname = %s%s%s) INNER JOIN pg_catalog.pg_attribute a ON (NOT a.attisdropped) AND a.attnum > 0 AND a.attrelid = c.oid) INNER JOIN pg_catalog.pg_type t ON t.oid = a.atttypid) LEFT OUTER JOIN pg_attrdef d ON a.atthasdef AND d.adrelid = a.attrelid AND d.adnum = a.attnum ORDER BY n.nspname, c.relname, attnum",
-				tablename,
-				ast_strlen_zero(schemaname) ? "" : "'", ast_strlen_zero(schemaname) ? "current_schema()" : schemaname, ast_strlen_zero(schemaname) ? "" : "'");
-		} else {
-			snprintf(sqlcmd, sizeof(sqlcmd), "SELECT a.attname, t.typname, a.attlen, a.attnotnull, d.adsrc, a.atttypmod FROM pg_class c, pg_type t, pg_attribute a LEFT OUTER JOIN pg_attrdef d ON a.atthasdef AND d.adrelid = a.attrelid AND d.adnum = a.attnum WHERE c.oid = a.attrelid AND a.atttypid = t.oid AND (a.attnum > 0) AND c.relname = '%s' ORDER BY c.relname, attnum", table);
-		}
-		/* Query the columns */
-		result = PQexec(conn, sqlcmd);
-		if (PQresultStatus(result) != PGRES_TUPLES_OK) {
-			pgerror = PQresultErrorMessage(result);
-			ast_log(LOG_ERROR, "Failed to query database columns: %s\n", pgerror);
-			PQclear(result);
-			unload_module();
-			ast_mutex_unlock(&pgsql_lock);
-			return AST_MODULE_LOAD_DECLINE;
-		}
-
-		rows = PQntuples(result);
-		if (rows == 0) {
-			ast_log(LOG_ERROR, "cdr_pgsql: Failed to query database columns. No columns found, does the table exist?\n");
-			PQclear(result);
-			unload_module();
-			ast_mutex_unlock(&pgsql_lock);
-			return AST_MODULE_LOAD_DECLINE;
-		}
-
-		/* Clear out the columns list. */
-		empty_columns();
-
-		for (i = 0; i < rows; i++) {
-			fname = PQgetvalue(result, i, 0);
-			ftype = PQgetvalue(result, i, 1);
-			flen = PQgetvalue(result, i, 2);
-			fnotnull = PQgetvalue(result, i, 3);
-			fdef = PQgetvalue(result, i, 4);
-			if (atoi(flen) == -1) {
-				/* For varchar columns, the maximum length is encoded in a different field */
-				flen = PQgetvalue(result, i, 5);
-			}
-
-			cur = ast_calloc(1, sizeof(*cur) + strlen(fname) + strlen(ftype) + 2);
-			if (cur) {
-				sscanf(flen, "%30d", &cur->len);
-				cur->name = (char *)cur + sizeof(*cur);
-				cur->type = (char *)cur + sizeof(*cur) + strlen(fname) + 1;
-				strcpy(cur->name, fname);
-				strcpy(cur->type, ftype);
-				if (*fnotnull == 't') {
-					cur->notnull = 1;
-				} else {
-					cur->notnull = 0;
-				}
-				if (!ast_strlen_zero(fdef)) {
-					cur->hasdefault = 1;
-				} else {
-					cur->hasdefault = 0;
-				}
-				AST_RWLIST_WRLOCK(&psql_columns);
-				AST_RWLIST_INSERT_TAIL(&psql_columns, cur, list);
-				AST_RWLIST_UNLOCK(&psql_columns);
-			}
-		}
-		PQclear(result);
-	} else {
-		pgerror = PQerrorMessage(conn);
-		ast_log(LOG_ERROR, "Unable to connect to database server %s.  CALLS WILL NOT BE LOGGED!!\n", pghostname);
-		ast_log(LOG_ERROR, "Reason: %s\n", pgerror);
-		connected = 0;
-		PQfinish(conn);
-		conn = NULL;
-	}
+	pgsql_connect();
 
 	ast_config_destroy(cfg);
-
 	ast_mutex_unlock(&pgsql_lock);
 	return 0;
+
+err:
+	ast_mutex_unlock(&pgsql_lock);
+	ast_config_destroy(cfg);
+	return -1;
 }
 
 static int load_module(void)
 {
 	ast_cli_register_multiple(cdr_pgsql_status_cli, sizeof(cdr_pgsql_status_cli) / sizeof(struct ast_cli_entry));
+
 	if (config_module(0)) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
-	return ast_cdr_register(name, ast_module_info->description, pgsql_log)
-		? AST_MODULE_LOAD_DECLINE : 0;
+
+	if (ast_cdr_register(name, ast_module_info->description, pgsql_log)) {
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	return AST_MODULE_LOAD_SUCCESS;
 }
 
 static int reload(void)
 {
-	return config_module(1);
+	if (config_module(1)) {
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+static int unload_module(void)
+{
+	if (ast_cdr_unregister(name)) {
+		return -1;
+	}
+
+	ast_cli_unregister_multiple(cdr_pgsql_status_cli, ARRAY_LEN(cdr_pgsql_status_cli));
+
+	pgsql_disconnect();
+
+	ast_free(pghostname);
+	ast_free(pgdbport);
+	ast_free(pgdbname);
+	ast_free(pgdbuser);
+	ast_free(pgpassword);
+	ast_free(table);
+	ast_free(pgappname);
+	ast_free(encoding);
+	ast_free(tz);
+
+	empty_columns();
+
+	return 0;
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PostgreSQL CDR Backend",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CDR_DRIVER,
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CDR_DRIVER,
+);
--- configs/samples/cdr_pgsql.conf.sample.orig2
+++ configs/samples/cdr_pgsql.conf.sample
@@ -8,9 +8,9 @@
 ;hostname=localhost
 ;port=5432
 ;dbname=asterisk
-;password=password
 ;user=postgres
-;appname=asterisk    ; Postgres application_name support (optional). Whitespace not allowed.
-;table=cdr		;SQL table where CDRs will be inserted
-;encoding=LATIN9	; Encoding of logged characters in Asterisk
+;password=password
+;table=cdr		; Table where CDRs will be inserted
+;appname=asterisk	; Postgres application_name support (optional). Whitespace not allowed.
+;encoding=UTF8		; Encoding of logged characters in Asterisk
 ;timezone=UTC		; Uncomment if you want datetime fields in UTC/GMT
